@@ -1,9 +1,7 @@
-"""A notebook manager that uses the local file system for storage."""
-
-# Copyright (c) IPython Development Team.
-# Distributed under the terms of the Modified BSD License.
+"""A notebook manager for IPython with MongoDB as the backend."""
 
 from tornado import web
+import os
 
 from io import StringIO
 import datetime
@@ -14,7 +12,8 @@ from mongodb_proxy import MongoProxy
 
 from IPython.html.services.notebooks.nbmanager import NotebookManager
 from IPython.nbformat import current
-from IPython.utils.traitlets import Unicode, Bool, TraitError
+from IPython.utils.traitlets import Unicode, CBool
+
 
 def sort_key(item):
     """Case-insensitive sorting."""
@@ -24,6 +23,7 @@ def sort_key(item):
 # Classes
 #-----------------------------------------------------------------------------
 
+
 class MongoNotebookManager(NotebookManager):
     #Useless variable that is required unfortunately
     notebook_dir = Unicode(u"", config=True)
@@ -43,12 +43,14 @@ class MongoNotebookManager(NotebookManager):
     notebook_collection = Unicode('notebooks', config=True,
         help="Defines the collection in mongodb in which to store the notebooks"
     )
-    
+
     checkpoint_collection = Unicode('checkpoints', config=True,
         help="The collection name in which to keep notebook checkpoints"
     )
 
-    
+    checkpoints_history = CBool('checkpoints_history', config=True,
+        help="Save all checkpoints or keep only last"
+    )
 
     def __init__(self, **kwargs):
         super(MongoNotebookManager, self).__init__(**kwargs)
@@ -56,26 +58,25 @@ class MongoNotebookManager(NotebookManager):
             self._conn = self._connect_server()
         else:
             self._conn = self._connect_replica_set()
-    
-    
+
     def get_notebook_names(self, path=''):
         """List all notebook names in the notebook dir and path."""
         path = path.strip('/')
         spec = {'path': path}
-        fields = {'name':1}
+        fields = {'name': 1}
         notebooks = list(self._connect_collection(self.notebook_collection).find(spec,fields))
         names = [n['name'] for n in notebooks]
         return names
 
     def path_exists(self, path):
         """Does the API-style path (directory) actually exist?
-        
+
         Parameters
         ----------
         path : string
             The path to check. This is an API path (`/` separated,
             relative to base notebook-dir).
-        
+
         Returns
         -------
         exists : bool
@@ -90,37 +91,10 @@ class MongoNotebookManager(NotebookManager):
         return count > 0
 
     def is_hidden(self, path):
-        """Does the API style path correspond to a hidden directory or file?
-        
-        Parameters
-        ----------
-        path : string
-            The path to check. This is an API path (`/` separated,
-            relative to base notebook-dir).
-        
-        Returns
-        -------
-        exists : bool
-            Whether the path is hidden.
-        
-        """
         #Nothing is hidden
         return False
 
     def notebook_exists(self, name, path=''):
-        """Returns a True if the notebook exists. Else, returns False.
-
-        Parameters
-        ----------
-        name : string
-            The name of the notebook you are checking.
-        path : string
-            The relative path to the notebook (with '/' as separator)
-
-        Returns
-        -------
-        bool
-        """
         path = path.strip('/')
         spec = {
             'path': path,
@@ -129,10 +103,7 @@ class MongoNotebookManager(NotebookManager):
         count = self._connect_collection(self.notebook_collection).find(spec).count()
         return count == 1
 
-    # TODO: Remove this after we create the contents web service and directories are
-    # no longer listed by the notebook web service.
     def list_dirs(self, path):
-        """List the directories for a given API style path."""
         path = path.strip('/')
         if path == '':
             prefix = ''
@@ -144,7 +115,7 @@ class MongoNotebookManager(NotebookManager):
                 '$regex': prefix + '*'
             }
         }
-        fields = {'name':1}
+        fields = {'name': 1}
         notebooks = list(self._connect_collection(self.notebook_collection).find(spec,fields))
         names = [n['name'].lstrip(prefix) for n in notebooks]
 
@@ -152,28 +123,24 @@ class MongoNotebookManager(NotebookManager):
         dirs = sorted(dirs, key=sort_key)
         return dirs
 
-
-    # TODO: Remove this after we create the contents web service and directories are
-    # no longer listed by the notebook web service.
     def get_dir_model(self, name, path=''):
-        """Get the directory model given a directory name and its API style path"""
         path = path.strip('/')
         spec = {
             'path': path,
             'name': name
         }
         fields = {
-            'lastModified':1,
+            'lastModified': 1,
             'created': 1
         }
         notebook = self._connect_collection(self.notebook_collection).find_one(spec,fields)
         if notebook == None:
             raise IOError('directory does not exist: %r' % (path + '|' + name))
 
-        last_modified = notebooks[0]['lastModified']
-        created = notebooks[0]['created']
+        last_modified = notebook[0]['lastModified']
+        created = notebook[0]['created']
         # Create the notebook model.
-        model ={}
+        model = {}
         model['name'] = name
         model['path'] = path
         model['last_modified'] = last_modified
@@ -182,44 +149,14 @@ class MongoNotebookManager(NotebookManager):
         return model
 
     def list_notebooks(self, path):
-        """Returns a list of dictionaries that are the standard model
-        for all notebooks in the relative 'path'.
-        
-        Parameters
-        ----------
-        path : str
-            the URL path that describes the relative path for the
-            listed notebooks
-        
-        Returns
-        -------
-        notebooks : list of dicts
-            a list of the notebook models without 'content'
-        """
         path = path.strip('/')
         notebook_names = self.get_notebook_names(path)
         notebooks = [self.get_notebook(name, path, content=False)
-                        for name in notebook_names if self.should_list(name)]
+                     for name in notebook_names if self.should_list(name)]
         notebooks = sorted(notebooks, key=sort_key)
         return notebooks
 
     def get_notebook(self, name, path='', content=True):
-        """ Takes a path and name for a notebook and returns its model
-        
-        Parameters
-        ----------
-        name : str
-            the name of the notebook
-        path : str
-            the URL path that describes the relative path for
-            the notebook
-            
-        Returns
-        -------
-        model : dict
-            the notebook model. If contents=True, returns the 'contents' 
-            dict in the model as well.
-        """
         path = path.strip('/')
         if not self.notebook_exists(name=name, path=path):
             raise web.HTTPError(404, u'Notebook does not exist: %s' % name)
@@ -229,7 +166,7 @@ class MongoNotebookManager(NotebookManager):
             'name': name
         }
         fields = {
-            'lastModified':1,
+            'lastModified': 1,
             'created': 1
         }
         if content:
@@ -240,7 +177,7 @@ class MongoNotebookManager(NotebookManager):
         last_modified = notebook['lastModified']
         created = notebook['created']
         # Create the notebook model.
-        model ={}
+        model = {}
         model['name'] = name
         model['path'] = path
         model['last_modified'] = last_modified
@@ -254,7 +191,6 @@ class MongoNotebookManager(NotebookManager):
         return model
 
     def save_notebook(self, model, name='', path=''):
-        """Save the notebook model and return the model with no content."""
         path = path.strip('/')
 
         if 'content' not in model:
@@ -272,9 +208,9 @@ class MongoNotebookManager(NotebookManager):
 
         # Save the notebook file
         nb = current.to_notebook_json(model['content'])
-        
+
         self.check_and_sign(nb, new_name, new_path)
-        
+
         if 'name' in nb['metadata']:
             nb['metadata']['name'] = u''
         try:
@@ -285,7 +221,7 @@ class MongoNotebookManager(NotebookManager):
                     'name': name
                 }
                 data = {
-                    '$set':{
+                    '$set': {
                         'content': f.getvalue(),
                         'lastModified': datetime.datetime.now(),
                     }
@@ -303,7 +239,6 @@ class MongoNotebookManager(NotebookManager):
         return model
 
     def update_notebook(self, model, name, path=''):
-        """Update the notebook's path and/or name"""
         path = path.strip('/')
         new_name = model.get('name', name)
         new_path = model.get('path', path).strip('/')
@@ -313,25 +248,24 @@ class MongoNotebookManager(NotebookManager):
         return model
 
     def delete_notebook(self, name, path=''):
-        """Delete notebook by name and path."""
         path = path.strip('/')
         spec = {
             'path': path,
             'name': name
         }
         fields = {
-            'name':1,
+            'name': 1,
         }
+
         notebook = self._connect_collection(self.notebook_collection).find_one(spec,fields)
-        if notebook == None:
+        if not notebook:
             raise web.HTTPError(404, u'Notebook does not exist: %s' % name)
-        
+
         # clear checkpoints
         self._connect_collection(self.checkpoint_collection).remove(spec)
         self._connect_collection(self.notebook_collection).remove(spec)
 
     def rename_notebook(self, old_name, old_path, new_name, new_path):
-        """Rename a notebook."""
         old_path = old_path.strip('/')
         new_path = new_path.strip('/')
         if new_name == old_name and new_path == old_path:
@@ -343,7 +277,7 @@ class MongoNotebookManager(NotebookManager):
             'name': new_name
         }
         fields = {
-            'name':1,
+            'name': 1,
         }
         notebook = self._connect_collection(self.notebook_collection).find_one(spec,fields)
         if notebook != None:
@@ -356,7 +290,7 @@ class MongoNotebookManager(NotebookManager):
                 'name': old_name
             }
             modify = {
-                '$set':{
+                '$set': {
                     'path': new_path,
                     'name': new_name
                 }
@@ -371,47 +305,50 @@ class MongoNotebookManager(NotebookManager):
             'name': old_name
         }
         modify = {
-            '$set':{
+            '$set': {
                 'path': new_path,
                 'name': new_name
             }
         }
         self._connect_collection(self.checkpoint_collection).update(spec, modify, multi=True)
-  
+
     # public checkpoint API
     def create_checkpoint(self, name, path=''):
-        """Create a checkpoint from the current state of a notebook"""
         path = path.strip('/')
         spec = {
             'path': path,
             'name': name
         }
+
         notebook = self._connect_collection(self.notebook_collection).find_one(spec)
+        chid = notebook['_id']
         del notebook['_id']
-        notebook = {'$set': notebook}
         cp_id = str(self._connect_collection(self.checkpoint_collection).find(spec).count())
-        last_modified = notebook['$set']["lastModified"]
-        spec['cp'] = cp_id
-        self._connect_collection(self.checkpoint_collection).update(spec, notebook, upsert=True)
-        
+
+        if self.checkpoints_history:
+            spec['cp'] = cp_id
+        else:
+            notebook['cp'] = cp_id
+            spec['id'] = chid
+
+        newnotebook = {'$set': notebook}
+
+        last_modified = notebook["lastModified"]
+        self._connect_collection(self.checkpoint_collection).update(spec, newnotebook, upsert=True)
+
         # return the checkpoint info
         return dict(id=cp_id, last_modified=last_modified)
-    
+
     def list_checkpoints(self, name, path=''):
-        """list the checkpoints for a given notebook"""
         path = path.strip('/')
-        cp_id = "checkpoint"
         spec = {
             'path': path,
             'name': name,
         }
         checkpoints = list(self._connect_collection(self.checkpoint_collection).find(spec))
         return [dict(id=c['cp'], last_modified=c['lastModified']) for c in checkpoints]
-        
-        
-    
+
     def restore_checkpoint(self, checkpoint_id, name, path=''):
-        """restore a notebook to a checkpointed state"""
         path = path.strip('/')
         spec = {
             'path': path,
@@ -422,17 +359,16 @@ class MongoNotebookManager(NotebookManager):
         checkpoint = self._connect_collection(self.checkpoint_collection).find_one(spec)
         print checkpoint
         if checkpoint == None:
-            raise web.HTTPError(404,
-                u'Notebook checkpoint does not exist: %s-%s' % (name, checkpoint_id)
+            raise web.HTTPError(
+                404, u'Notebook checkpoint does not exist: %s-%s' % (name, checkpoint_id)
             )
         del spec['cp']
         del checkpoint['cp']
         del checkpoint['_id']
         checkpoint = {'$set': checkpoint}
         self._connect_collection(self.notebook_collection).update(spec, checkpoint, upsert=True)
-    
+
     def delete_checkpoint(self, checkpoint_id, name, path=''):
-        """delete a notebook's checkpoint"""
         path = path.strip('/')
         spec = {
             'path': path,
@@ -445,20 +381,19 @@ class MongoNotebookManager(NotebookManager):
                 u'Notebook checkpoint does not exist: %s%s-%s' % (path, name, checkpoint_id)
             )
         self._connect_collection(self.checkpoint_collection).remove(spec)
-    
+
     def info_string(self):
         return "Serving notebooks from mongodb"
 
     def get_kernel_path(self, name, path='', model=None):
-        """ Return the path to start kernel in """
         return os.path.join(self.notebook_dir, path)
 
     #mongodb related functions
     def _connect_server(self):
-        return MongoProxy(pymongo.MongoClient(self.mongo_uri));
+        return MongoProxy(pymongo.MongoClient(self.mongo_uri))
 
     def _connect_replica_set(self):
-        return MongoProxy(pymongo.MongoReplicaSetClient(self.mongo_uri, self._replicaSet));
+        return MongoProxy(pymongo.MongoReplicaSetClient(self.mongo_uri, self._replicaSet))
 
     def _connect_collection(self, collection):
         if not self._conn.alive():
@@ -466,4 +401,4 @@ class MongoNotebookManager(NotebookManager):
                 self._conn = self._connect_server()
             else:
                 self._conn = self._connectReplicaSet()
-        return self._conn[self.database_name][collection]
\ No newline at end of file
+        return self._conn[self.database_name][collection]
